from worlddata.APISections.base import WorldDataBase


class WorldDataTimeSeries(WorldDataBase):

    def time_series(self, sector, sub_sector, super_region, source, trend_ids):
        return self.call_api_post("time-series", sector=sector, sub_sector=sub_sector, super_region=super_region,
                                  source=source, trend_ids=trend_ids)



import re
from operator import itemgetter

from worlddata.APIExceptions.WorldDataExceptions import WorldDataException
from worlddata.APISections.base import WorldDataBase


class WorldDataAdvanceSearch(WorldDataBase):

    def advance_search(self, search_text, sector, sub_sector, super_region, source, size, offset, **kwargs):
        return self.call_api_post("advanced-search", search_text=search_text, sector=sector, sub_sector=sub_sector,
                                  super_region=super_region, source=source, size=size, offset=offset, kwargs=kwargs)

    def advance_search_attributes(self, search_text, sector, sub_sector, super_region, source, **kwargs):
        return self.call_api_post("attributes", search_text=search_text, sector=sector, sub_sector=sub_sector,
                                  super_region=super_region, source=source, kwargs=kwargs)

    def news_sentiment(self, search_text, sector, sub_sector, super_region, source, type, **kwargs):
        return self.call_api_post("news/sentiment", search_text=search_text, sector=sector, sub_sector=sub_sector,
                                  super_region=super_region, source=source, type=type, kwargs=kwargs)

    def news_entity_mapping(self, search_text, sector, sub_sector, super_region, source, **kwargs):
        news_json = self.call_api_post("news/entity-mappings", search_text=search_text, sector=sector,
                                       sub_sector=sub_sector,
                                       super_region=super_region, source=source, kwargs=kwargs)
        text = ''
        for item in news_json:
            for key in item:
                if item[key] and len(item[key]) > 0:
                    if len(text) == 0:
                        text = item[key][0].replace(';', ',')
                    else:
                        text = text + ',' + item[key][0].replace(';', ',')

        lines = re.split("\s|(?<!\d)[,.](?!\d)", text)

        str2 = []
        data = []
        # loop till string values present in list str
        for i in lines:

            # checking for the duplicacy
            if i not in str2:
                # insert value in str2
                str2.append(i)

        for i in range(0, len(str2)):
            if len(str2[i]) > 1:
                obj = {
                    'name': str2[i],
                    'weight': lines.count(str2[i])
                }
                data.append(obj)

        data.sort(key=itemgetter('weight'), reverse=True)

        data = data[0:25:]

        data_list = []

        for fromIndex, fromItem in enumerate(data):
            for toIndex, toItem in enumerate(data):
                if fromIndex == toIndex:
                    continue

                data_item = {'from': fromItem['name'], 'to': toItem['name'], 'weight': 0};
                for item in news_json:
                    for key in item:
                        join_list = ' '.join(item[key])
                        if join_list and len(join_list) > 0 and join_list.find(data_item['from']) > -1:
                            str_reg_ex_pattern = '\\b' + data_item['to'] + '\\b'
                            rg = re.compile(str_reg_ex_pattern)
                            count = len(rg.findall(join_list))
                            data_item['weight'] += count

                data_list.append(data_item)
        return data_list

    def news_search(self, search_text, size, offset, **kwargs):
        search_buckets = set()
        outputs = search_text.split()
        for search in outputs:
            search_buckets.update(search.split(','))

        if len(search_buckets) > 30:
            raise WorldDataException("Search term should be less than or equal to 30")

        search_text = ",".join(outputs)
        return self.call_api_post("advanced-search", search_text=search_text, sector='NEWS AI, SENTIMENTS', size=size,
                                  offset=offset, kwargs=kwargs)



from worlddata.APISections.base import WorldDataBase


class WorldDataSearch(WorldDataBase):

    def search(self, search_text, **kwargs):
        return self.call_api_post("search", search_text=search_text, kwargs=kwargs)
    


# -*-coding:utf-8-*-

from worlddata.APISections.advance_search import WorldDataAdvanceSearch
from worlddata.APISections.search import WorldDataSearch
from worlddata.APISections.time_series import WorldDataTimeSeries


class WorldData(
    WorldDataSearch,
    WorldDataTimeSeries,
    WorldDataAdvanceSearch
):
    pass



import json

import requests

from worlddata.APIExceptions.WorldDataExceptions import WorldDataException


class WorldDataBase:
    API_path = "/api/rest/zdaly/trends/"

    def __init__(
        self,
        auth_token,
        server_url="https://zdaly.com",
        ssl_verify=True,
        proxies=None,
        timeout=60,
        session=None,
        client_certs=None,
    ):
        self.headers = {}
        self.server_url = server_url
        self.proxies = proxies
        self.ssl_verify = ssl_verify
        self.cert = client_certs
        self.timeout = timeout
        self.req = session or requests
        if not auth_token:
            raise WorldDataException("API Key not provided")
        self.headers["api-key"] = auth_token

        self.headers['Content-type'] = 'application/json'
        self.headers['Accept'] = 'application/json'

    @staticmethod
    def __reduce_kwargs(kwargs):
        if "kwargs" in kwargs:
            for arg in kwargs["kwargs"].keys():
                kwargs[arg] = kwargs["kwargs"][arg]

            del kwargs["kwargs"]
        return kwargs

    def call_api_delete(self, method):
        url = self.server_url + self.API_path + method
        response = self.req.delete(
            url,
            headers=self.headers,
            verify=self.ssl_verify,
            cert=self.cert,
            proxies=self.proxies,
            timeout=self.timeout,
        )
        self.parse_response_codes(response)
        json_data = response.json()
        return json_data['responseEntity']

    def call_api_get(self, method, **kwargs):
        args = self.__reduce_kwargs(kwargs)
        url = self.server_url + self.API_path + method
        # convert to key[]=val1&key[]=val2 for args like key=[val1, val2], else key=val
        params = "&".join(
            "&".join(i + "[]=" + j for j in args[i])
            if isinstance(args[i], list)
            else i + "=" + str(args[i])
            for i in args
        )
        response = self.req.get(
            "%s?%s" % (url, params),
            headers=self.headers,
            verify=self.ssl_verify,
            cert=self.cert,
            proxies=self.proxies,
            timeout=self.timeout,
        )
        self.parse_response_codes(response)
        json_data = response.json()
        return json_data['responseEntity']

    def call_api_post(self, method, files=None, use_json=None, **kwargs):
        reduced_args = self.__reduce_kwargs(kwargs)
        response = self.req.post(
            self.server_url + self.API_path + method,
            json=reduced_args,
            files=files,
            headers=self.headers,
            verify=self.ssl_verify,
            cert=self.cert,
            proxies=self.proxies,
            timeout=self.timeout,
        )
        self.parse_response_codes(response)
        json_data = response.json()
        return json_data['responseEntity']

    @staticmethod
    def parse_response_codes(response):
        """Parses the reponse code and raises a APIConnection error if not 200"""
        if response.status_code == 200:
            return

        json_data = response.json()

        if response.status_code == 400:
            raise WorldDataException(json_data['responseMessage'])

        if response.status_code == 401:
            raise WorldDataException(json_data['responseMessage'])

        if response.status_code == 403:
            raise WorldDataException(json_data['responseMessage'])

        if response.status_code == 404:
            raise WorldDataException("The URI requested is invalid or the requested resource does not exist")

        if response.status_code == 429:
            raise WorldDataException("You are making requests too quickly, slow down and try again.")

        if response.status_code == 500:
            raise WorldDataException("Something unexpected occurred.")

        if response.status_code == 502:
            raise WorldDataException("WorlData service is down")

        if response.status_code == 503:
            raise WorldDataException("WorldData service is up but overloaded with requests")

        else:
            raise WorldDataException(json_data['responseMessage'])



# -*-coding:utf-8-*-

from setuptools import setup

setup(
    name="worlddata",
    version="0.0.3",
    packages=[
        "worlddata",
        "worlddata.APIExceptions",
        "worlddata.APISections",
    ],
    url="https://github.com/worlddata-ai/python-api",
    license="WorldData",
    author="Akash Doifode",
    author_email="akash.doifode@worlddata.ai",
    description="Python API wrapper for WorldData.AI",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    install_requires=("requests",),
)
